% Run profile Cholesky on a random SPD test matrix.
%
% format is 1,2,3 for the memory layout required (see comments in
% profchol.m). prof is an integer row matrix with the profile (prof(i)
% is the row index of the first nonzero element in column i.
%
% The routine runs both standard dense and profile Cholesky, solves a
% random system, and prints run times and solution residuals.

function tst_profchol(format,prof)
   eps = 0;
   % read profile, or generate a random one
   n = 5;
   if nargin<2, prof = []; end
   if max(size(prof))==1, n = prof; prof = []; end
   if isempty(prof)
      % prof = max(1,ceil([1:n]-sqrt(n*abs(randn(1,n))))); 
      prof = max(1,[1:n]-1-floor(rand(1,n).*[1:n]/4));
   end
   n = size(prof,2);
   % prof

   % build a random SPD test matrix A in unpacked form
   A = zeros(n);
   for i=1:n 
      v = randn(i-prof(i)+1,1); 
      A(prof(i):i,i) = v;
      A(i,prof(i):i) = v';
      A(i,i) = abs(A(i,i));
   end
   A = A + diag(sum(abs(A)));

   b = randn(n,1);
   % [A,b]
   t0=cputime;
   R2 = chol(A);
   x1 = R2' \ b;
   x2 = R2 \ x1;
   % [R2,x1,x2,R2'*R2*x2-b]
   dt=cputime-t0
   norm(R2'*R2*x2-b)
   
   % pack A into R
   if format == 1		% unpacked format
      R = A;
      rprof = prof;
   elseif format == 2		% b x n band matrix format
      bw = max([1:n]-prof)+1;
      rprof = prof-[1:i]+bw;
      R = zeros(bw,n);
      for i=1:n
	 R(rprof(i):bw,i) = A(prof(i):i,i);
      end
   elseif format == 3		% fully packed format
      rprof = [0,cumsum([2:n+1]-prof,2)];
      R = zeros(rprof(n+1),1);
      for i=1:n
	 % i,rprof(i)+1:rprof(i+1),prof(i):i
	 R(rprof(i)+1:rprof(i+1)) = A(prof(i):i,i);
      end
   else
      error('bad matrix format');
   end
   % R,rprof
   t0=cputime;
   [R1,rprof1] = profchol(format,R,rprof,eps);
   b1 = proffsub(format,R1,rprof1,b);
   b2 = profbsub(format,R1,rprof1,b1);
   dt=cputime-t0
   % R1, [b1,b2,b2-x2]
   norm(b2-x2)

%end
   
